@@ -1,19 +1,45 @@
 """Available slide building classes."""
 
+import json
 import os
 
 from docutils import nodes
-from sphinx.theming import Theme
+import sphinx
 from sphinx.builders.html import (
+    SingleFileHTMLBuilder,
     StandaloneHTMLBuilder,
     DirectoryHTMLBuilder,
 )
-from sphinx.util import copy_static_entry
+try:
+    from sphinx.util.fileutil import copy_asset_file
+except ImportError:
+    copy_asset_file = None
+    from sphinx.util import copy_static_entry
 
 from hieroglyph import writer
 from hieroglyph import directives
 
 
+if sphinx.version_info < (1, 6, 0):
+    from sphinx.theming import Theme
+
+    class HTMLThemeFactory:
+        """Compatibility shim to make old versions of Sphinx act more like 1.6+."""
+        def __init__(self, app):
+            self.app = app
+
+        def load_additional_themes(self, paths):
+            Theme.init_themes(self.app.confdir, paths)
+
+        def create(self, themename):
+            return Theme(themename)
+
+    Theme.get_config = Theme.get_confstr
+    Theme.get_theme_dirs = Theme.get_dirchain
+else:
+    from sphinx.theming import HTMLThemeFactory
+
+
 def building_slides(app):
     """Returns True if building Slides."""
 
@@ -22,10 +48,13 @@ def building_slides(app):
 
 class AbstractSlideBuilder(object):
 
+    format = 'slides'
     add_permalinks = False
+    default_translator_class = writer.SlideTranslator
 
     def init_translator_class(self):
-        self.translator_class = writer.SlideTranslator
+        """Compatibility shim to support versions of Sphinx prior to 1.6."""
+        self.translator_class = self.default_translator_class
 
     def get_builtin_theme_dirs(self):
 
@@ -38,15 +67,16 @@ class AbstractSlideBuilder(object):
 
         return self.config.slide_theme, self.config.slide_theme_options
 
+    def get_theme_options(self):
+        """Return a dict of theme options, combining defaults and overrides."""
+
+        overrides = self.get_theme_config()[1]
+        return self.theme.get_options(overrides)
+
     def init_templates(self):
-        Theme.init_themes(self.confdir,
-                          self.get_builtin_theme_dirs() +
-                          self.config.slide_theme_path,
-                          warn=self.warn)
         themename, themeoptions = self.get_theme_config()
 
         self.create_template_bridge()
-
         self._theme_stack = []
         self._additional_themes = []
 
@@ -65,18 +95,29 @@ class AbstractSlideBuilder(object):
             (self.theme, self.theme_options)
         )
 
-        self.theme = Theme(themename)
+        theme_factory = HTMLThemeFactory(self.app)
+        theme_factory.load_additional_themes(self.get_builtin_theme_dirs() + self.config.slide_theme_path)
+
+        self.theme = theme_factory.create(themename)
         self.theme_options = themeoptions.copy()
         self.templates.init(self, self.theme)
+        self.templates.environment.filters['json'] = json.dumps
 
-        self._additional_themes.append(self.theme)
+        if self.theme not in self._additional_themes:
+            self._additional_themes.append(self.theme)
 
     def pop_theme(self):
         """Disable the most recent theme, and restore its predecessor."""
 
         self.theme, self.theme_options = self._theme_stack.pop()
 
-        self.templates.init(self, self.theme)
+    def prepare_writing(self, docnames):
+
+        super(AbstractSlideBuilder, self).prepare_writing(docnames)
+
+        # override items in the global context if needed
+        if self.config.slide_title:
+            self.globalcontext['docstitle'] = self.config.slide_title
 
     def get_doc_context(self, docname, body, metatags):
 
@@ -86,7 +127,7 @@ class AbstractSlideBuilder(object):
 
         if self.theme:
             context.update(dict(
-                style=self.theme.get_confstr('theme', 'stylesheet'),
+                style=self.theme.get_config('theme', 'stylesheet'),
             ))
 
         return context
@@ -111,11 +152,10 @@ class AbstractSlideBuilder(object):
         super(AbstractSlideBuilder, self).post_process_images(doctree)
 
         # figure out where this doctree is in relation to the srcdir
-        relative_base = ['..'] * len(
-            os.path.dirname(
-                doctree.attributes.get('source')[len(self.srcdir) + 1:]
-                ).split('/')
-            )
+        relative_base = (
+            ['..'] *
+            doctree.attributes.get('source')[len(self.srcdir) + 1:].count('/')
+        )
 
         for node in doctree.traverse(nodes.image):
 
@@ -138,13 +178,16 @@ class AbstractSlideBuilder(object):
         ctx = self.globalcontext.copy()
         ctx.update(self.indexer.context_for_searchtool())
 
-        for theme in self._additional_themes:
+        staticdir = os.path.join(self.outdir, '_static')
+        for theme in self._additional_themes[1:]:
 
             themeentries = [os.path.join(themepath, 'static')
-                            for themepath in theme.get_dirchain()[::-1]]
+                            for themepath in theme.get_theme_dirs()[::-1]]
             for entry in themeentries:
-                copy_static_entry(entry, os.path.join(self.outdir, '_static'),
-                                  self, ctx)
+                if copy_asset_entry:
+                    copy_asset_entry(entry, staticdir, ctx)
+                else:
+                    copy_static_entry(entry, staticdir, self, ctx)
 
         return result
 
@@ -183,9 +226,24 @@ class SlideBuilder(AbstractSlideBuilder, StandaloneHTMLBuilder):
     name = 'slides'
 
 
+class SingleFileSlideBuilder(AbstractSlideBuilder, SingleFileHTMLBuilder):
+    """This is the single file Slide HTML builder.
+
+    Its output is a directory with a single HTML file, along with the
+    needed style sheets and JavaScript.
+
+    Its name is ``singlefile-slides``.
+
+    """
+
+    name = 'singlefile-slides'
+    default_translator_class = writer.SingleFileSlideTranslator
+
+
 class AbstractInlineSlideBuilder(object):
 
     name = 'inlineslides'
+    default_translator_class = writer.BaseSlideTranslator
 
     def __init__(self, *args, **kwargs):
         super(AbstractInlineSlideBuilder, self).__init__(*args, **kwargs)
@@ -204,9 +262,6 @@ class AbstractInlineSlideBuilder(object):
 
         self.css_files.append('_static/slides.css')
 
-    def init_translator_class(self):
-        self.translator_class = writer.BaseSlideTranslator
-
 
 class DirectoryInlineSlideBuilder(
         AbstractInlineSlideBuilder,
