@@ -91,12 +91,14 @@ class NextSlideDirective(Directive):
     final_argument_whitespace = True
     option_spec = {
         'increment': directives.flag,
+        'classes': directives.class_option,
     }
 
     def run(self):
 
         node = nextslide(**self.options)
         node.args = self.arguments
+        node.state = self.state
         node.document = self.state.document
         set_source_info(self, node)
 
@@ -143,7 +145,7 @@ class TransformNextSlides(Transform):
         parent_title_node = node.parent.next_node(nodes.title)
         nextslide_info = getattr(
             parent_title_node, 'nextslide_info',
-            (parent_title_node.astext(), 1),
+            (parent_title_node.deepcopy().children, 1),
         )
         nextslide_info = (
             nextslide_info[0],
@@ -151,17 +153,26 @@ class TransformNextSlides(Transform):
         )
 
         if node.args:
-            title_text = node.args[0]
-        elif 'increment' in node.attributes:
-            # autogenerating titles;
-            title_text = '%s (%d)' % nextslide_info
+            textnodes, messages = node.state.inline_text(
+                node.args[0],
+                1,
+            )
+            new_title = nodes.title(node.args[0], '', *textnodes)
+
         else:
-            title_text = nextslide_info[0]
+
+            title_nodes = nextslide_info[0][:]
+
+            if 'increment' in node.attributes:
+                title_nodes.append(
+                    nodes.Text(' (%s)' % nextslide_info[1])
+                )
+
+            new_title = nodes.title(
+                '', '',
+                *title_nodes
+            )
 
-        new_title = nodes.title(
-            '',
-            title_text,
-        )
         new_title.nextslide_info = nextslide_info
         return new_title
 
@@ -171,7 +182,7 @@ class TransformNextSlides(Transform):
 
         if (not building_slides or
                 not node.parent.children[index+1:]):
-            node.replace_self([])
+            node.parent.replace(node, [])
 
             # nothing else to do
             return
@@ -192,6 +203,10 @@ class TransformNextSlides(Transform):
         new_section.extend(new_children)
         self.document.set_id(new_section)
 
+        # add classes, if needed
+        if node.get('classes'):
+            new_section['classes'].extend(node.get('classes'))
+
         # attach the section and delete the nextslide node
         grandparent.insert(insertion_point, new_section)
         del node.parent[index]
